-------------------------------------------
--- Author: Ketho (EU-Boulderfist)		---
--- License: Public Domain				---
--- Created: 2011.11.27					---
--- Version: 0.8.1 [2012.06.26]			---
-------------------------------------------
--- Curse			http://www.curse.com/addons/wow/guildleveltooltip
--- WoWInterface	http://www.wowinterface.com/downloads/info20527-GuildLevelTooltip.html
--- Request			http://www.curse.com/forums/games/world-of-warcraft/addons/148517-got-a-request

-- Thanks to Cowmonster/Isoloedlk (US-Khaz Modan) for a great amount of help with the inspect code

local NAME = ...
local VERSION = "0.8.1"

local db, realmdb

local format = format

local f = CreateFrame("Frame")

	--------------
	--- Events ---
	--------------

function f:ADDON_LOADED(name)
	if name == NAME then
		GuildLevelTooltipDB = setmetatable(GuildLevelTooltipDB or {}, {
			__index = function(t, k)
				local v = {}
				rawset(t, k, v)
				return v
			end
		})
		GuildLevelTooltipDB.version = VERSION
		realmdb = GuildLevelTooltipDB[GetRealmName()]
		self:UnregisterEvent("ADDON_LOADED")
	end
end

local newUnit = "target"

-- Blizzard_InspectUI not loaded -> safe to NotifyInspect()
-- Blizzard_InspectUI loaded, hidden -> fix with InspectFrame.unit = "player", then NotifyInspect()
-- Blizzard_InspectUI loaded, shown -> don't NotifyInspect(), also not needed anyway

local function Notify(unit)
	if CanInspect(unit) and time() > (f.cd or 0) then
		-- NotifyInspect() is throttled by 6 calls for every 10 sec
		f.cd = time() + 2; newUnit = unit
		
		if InspectFrame then
			if not InspectFrame:IsShown() then
				InspectFrame.unit = "player"
				NotifyInspect(unit)
			end
		else
			NotifyInspect(unit)
		end
	end
end

function f:UPDATE_MOUSEOVER_UNIT()
	Notify("mouseover")
end

function f:PLAYER_TARGET_CHANGED()
	Notify("target")
end

local function IsCrossRealm(unit, save)
	local _, realm = UnitName(unit)
	return realm and GuildLevelTooltipDB[realm] or realmdb
end

function f:INSPECT_READY(guid)
	-- Blizzard_InspectUI is loaded and shown
	if InspectFrame and InspectFrame:IsShown() then
		newUnit = "target" -- unit is mostly "target" on manual inspect
	end
	
	-- INSPECT_READY passes the unit's guid instead of the unit id
	if UnitGUID(newUnit) == guid then
		local name = GetGuildInfo(newUnit)
		if not name then return end
		
		local _, level, xp, members = GetInspectGuildInfo(newUnit)
		if level == 0 then return end
		
		db = IsCrossRealm(newUnit)
		
		local pendingUpdate
		if not db[name] then
			pendingUpdate = true
		end
		
		db[name] = db[name] or {}
		db[name][1] = level
		db[name][2] = members
		db[name][3] = time()
		db[name][4] = date("%Y.%m.%d %H:%M")
		--db[name][5] = xp -- maybe useful if you want to closely see which guild levels faster
		
		if pendingUpdate then
			f.OnTooltipSetUnit(GameTooltip)
		end
	end
end

function f:OnEvent(event, ...)
	self[event](self, ...)
end

local events = {
	"ADDON_LOADED",
	"UPDATE_MOUSEOVER_UNIT",
	"PLAYER_TARGET_CHANGED",
	"INSPECT_READY"
}

for _, v in ipairs(events) do
	f:RegisterEvent(v)
end

f:SetScript("OnEvent", f.OnEvent)

	------------
	--- Hook ---
	------------

-- GameTooltip gets passed, self refers to that instead
function f:OnTooltipSetUnit()
	local _, unit = self:GetUnit()
	if not unit then return end
	
	db = IsCrossRealm(unit)
	
	local guild = db[GetGuildInfo(unit)]
	if not guild then return end
	
	self:AddLine(format(GUILD_LEVEL, guild[1]))
	self:AddLine(format(INSPECT_GUILD_NUM_MEMBERS, guild[2]))
	self:Show() -- update tooltip
end

GameTooltip:HookScript("OnTooltipSetUnit", f.OnTooltipSetUnit)
